@@ -1,7 +1,7 @@
 #!/usr/bin/env -S guile -e start -s
 !#
 ;;; coding: utf-8
-;;; $Id: debian-copyrighter.scm,v 1.40 2022/12/16 17:46:04 fpp Exp fpp $
+;;; $Id: debian-copyrighter.scm,v 1.50 2023/01/12 23:48:09 fpp Exp fpp $
 ;;; Copyright © 2022 Frank Pursel <frank.pursel@gmail.com>
 
  ;; The debian-copyrighter is free software; you can redistribute it and/or modify it
@@ -27,18 +27,15 @@
  (ice-9 getopt-long)
  (ice-9 format)
  (ice-9 ftw)
+ (ice-9 match)
  (srfi srfi-1)
  (srfi srfi-9)
  (srfi srfi-9 gnu)
  (srfi srfi-11)
  (ice-9 regex)
  (ice-9 rdelim)
- (srfi srfi-64))
-
-;; Default project data.
-(define project-name "guix")
-(define project-license 'GPL-3.0+)
-(define project-source "https://ftp.gnu.org/gnu/guix")
+ (srfi srfi-64)
+ (srfi srfi-26))
 
 (define-record-type <licensed-file>
   (make-licensed-file full-name dir license copyright-list)
@@ -47,6 +44,11 @@
   (dir licensed-file-dir set-licensed-file-dir!)
   (license licensed-file-license set-licensed-file-license!)
   (copyright-list licensed-file-copyright-list set-licensed-file-copyright-list!))
+
+(define (licensed-file-license->string rec)
+  "license must be a symbol for processing but then a string at output."
+  (symbol->string (licensed-file-license rec)))
+
 (define-record-type <copyright>
   (make-copyright tag year-list)
   copyright?
@@ -99,7 +101,7 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 	  (#t  (let* ((sorted-num-list (sort yr-lst <=))
 		      (min-yr (car sorted-num-list))
 		      (max-yr (car (reverse sorted-num-list))))
-		 (format #f "~a,~a" min-yr max-yr)))
+		 (format #f "~a-~a" min-yr max-yr)))
 	  )
     )
   )
@@ -174,6 +176,11 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
    '(GPL-3.0+ "same license as the( GNU)* guix package")
    '(GPL-2.0+ "GNU General Public License" "version 2" "any later")
    '(GPL-2or3 "GNU General Public License" "version 2" "version 3")
+   '(GPL-2.0 "GNU " "Public License" "Version 2")
+   '(GPL-2.0 "GNU General Public" "License version 2")
+   '(GPL-2.0 "GNU GPL version 2")
+   '(GPL "GNU General Public")
+   '(BEER-WARE "BEER-WARE LICENSE")
    '(LGPL-2.1+ "GNU Lesser General Public"
 	       "version 2.1" "any later")
    '(LGPL-3.0+ "GNU Lesser General Public"
@@ -186,6 +193,8 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 		  "1" "2" "AS IS")
    '(BSD-3-Clause "Redistribution" "provided that the following conditions are met:"
 		  "Neither the name of the author nor the names of contributors")
+   '(Modified-BSD-4 "Redistribution and use" "software must not be misrepresented" "altered source versions must be plainly marked")
+   '(BSD-style "BSD-style license")
    '(Expat "Permission is hereby granted, free of charge, to any person obtaining")
    '(MIT "MIT License")
    '(PERMISSIVE "This file can be copied and used freely without restrictions."
@@ -207,7 +216,10 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
       (any (lambda (l-of-f) (regexp-exec compiled-re l-of-f)) lof)))
   )
 
-(define lof-contains-license? (file-contains-icase? " license "))
+(define (lof-contains-license? lof)
+  (or ((file-contains-icase? " license") lof)
+      ((file-contains-icase? " Redistribution and use ") lof)
+     ))
 
 (define (dir-from-full-file-name ffn)
   (string-join (reverse (cdr (reverse (string-split ffn #\/)))) "/"))
@@ -251,10 +263,17 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 			 ") "))
 		  regexp/icase))
 
+(define copyr-re-i (make-regexp
+		  (string-concatenate
+		   (list "copyright( \\([Cc]\\)| "
+			 (list->string (list #\x00A9))
+			 ")* "))
+		  regexp/icase))
+
 (define (copyrights-of-aux lof)
   (if (null? lof)
       '()
-      (let ((match? (regexp-exec copyr-re (car lof))))
+      (let ((match? (regexp-exec copyr-re-i (car lof))))
 	(if match?
 	    (cons
 	     (make-copyright-from match?)
@@ -320,6 +339,65 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
     (filter empty-copyr? copyr-list))
   )
 
+(define (spdx-license? lof)
+  (spdx-license-auxx lof #f 15))
+
+(define spdx-raw-re (make-regexp "([-[:alpha:]]+):[:space:]*(.*)"))
+
+(define spdx-lic-raw-re
+  (make-regexp "SPDX-License-Identifier:[[:space:]]*(.*)"))
+(define spdx-lic-dquot-re
+  (make-regexp "SPDX-License-Identifier:[[:space:]]*\"(.*)\""))
+(define spdx-lic-cparen-re
+  (make-regexp "SPDX-License-Identifier:[[:space:]]*\\((.*)\\)"))
+
+(define prune-end-comment-re
+  (make-regexp "(.*)[[:space:]]*\\*/"))
+
+(define (spdx-license-identifier-value line)
+  (let ((value+end
+	 (cond
+	  ((regexp-exec spdx-lic-cparen-re line) => (cut match:substring <> 1))
+	  ((regexp-exec spdx-lic-dquot-re line) => (cut match:substring <> 1))
+	  ((regexp-exec spdx-lic-raw-re line) => (cut match:substring <> 1))
+	  (else #f))))
+    (if value+end
+	(let ((end-comment-match? (regexp-exec prune-end-comment-re value+end)))
+	  (string->symbol
+	   (string-trim-right (if end-comment-match?
+				  (match:substring end-comment-match? 1)
+				  value+end))))
+	#f
+	)
+    )
+  )
+
+(define (spdx-license-auxx lof res lines-to-check)
+  (if (or (null? lof) res (< lines-to-check 1))
+      res
+      (spdx-license-auxx
+       (cdr lof)
+       (spdx-license-identifier-value (car lof))
+       (- lines-to-check 1)
+       )
+      )
+  )
+  
+
+;;; Unified licensing policy
+(define (licensing? text-lic spdx-lic)
+  (cond ((not (or text-lic spdx-lic)) ;; No license found
+	 #f)
+	((and text-lic spdx-lic) ;; Both licenses found
+	 (if (eq? text-lic spdx-lic)
+	     text-lic
+	     (if (eq? text-lic 'UNKNOWN)
+		 spdx-lic
+		 text-lic)))
+	((or text-lic spdx-lic) => identity) 
+	))
+
+;;; Revised base-cut-leaf to incorporate SPDX license information if available.
 (define (base-cut-leaf base-dir exclusion-func)
   (let ((trim-base (lambda (name)
 		     (let ((lbd (string-length base-dir)))
@@ -332,31 +410,53 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
     (lambda (name stat result)
       (let ((res result))
 	(if (exclusion-func name stat result) ;;#t or (string-match ".*\\.(scm|po)$" name)
-	    (let ((lof (lines-of-file name))
-		  )
-	      (if (lof-contains-license? lof)
-		  (hash-set! res name (make-licensed-file
-				       (trim-base name)
-				       (dir-from-full-file-name name)
-				       (license-of lof license-tests)
-				       (filter-empty-copyrights
-					(copyrights-of lof (basename name)))))
-		  )
+	    (let* ((lof (lines-of-file name))
+		   (text-lic? (and (lof-contains-license? lof)
+				  (license-of lof license-tests)
+				  )
+			      )
+		   (spdx-lic? (spdx-license? lof))
+		   )
+	      (hash-set! res name (make-licensed-file
+				   (trim-base name)
+				   (dir-from-full-file-name name)
+				   (licensing? text-lic? spdx-lic?)
+				   (filter-empty-copyrights
+				    (copyrights-of lof (basename name)))))
+		  
 	      ))
 	res
 	))))
 
+
 (define (up name stat results) results)
 (define (down name stat results) results)
 (define (skip name stat results) results)
 (define (err name stat errno results) results)
 
+(define (analyze-single-file fn)
+  "Provide the analysis for a single file.  For testing."
+  (let* ((lof (lines-of-file fn))
+	 (text-lic? (and (lof-contains-license? lof)
+			 (license-of lof license-tests)))
+	 (spdx-lic? (spdx-license? lof))
+	 (copyrights (copyrights-of lof (basename fn))))
+    (format #t "Prose-license: ~a~%SPDX-license: ~a~%Arbit: ~a~%Copyrights: ~a~%"
+	    text-lic?
+	    spdx-lic?
+	    (licensing? text-lic? spdx-lic?)
+	    (string-join
+	     (map (lambda (x) (format #f "~a" x)) copyrights)
+	     "\n"))
+    ))
+
 (define (hash-for-dir-tree dir)
   "Main entry point to scan a directory tree and return a hash of all files with license and copyright analyses."
   (let ((leaf (base-cut-leaf dir (const #t))))
     (file-system-fold enter? (base-cut-leaf dir (const #t)) down up skip err
 		      (make-hash-table 1200)
-		      dir)))
+		      dir
+		      stat)))
 
 (define (enter-w-exclusions? excl-list)
   "Provided a set of regular-expressions return an enter? function. Delimited defined by excl-delim-char"
@@ -375,7 +475,8 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 	(enter? (enter-w-exclusions? regexp-list)))
     (file-system-fold enter? leaf down up skip err
 		      (make-hash-table 1200)
-		      dir)))
+		      dir
+		      stat)))
 
 (define (problems-to-investigate dir)
   (let ((loc-hash (hash-for-dir-tree dir)))
@@ -420,7 +521,7 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 	(down (lambda  (name stat result) result))
 	(err (lambda  (name stat errno result) result))
 	)
-    (file-system-fold entry? leaf down up skip err '() base-dir)))
+    (file-system-fold entry? leaf down up skip err '() base-dir stat)))
 
 (define (single-licensed-dir? dir hash)
   (let ((lic-lst (license-freq dir hash)))
@@ -579,7 +680,7 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 					     (copyright-tag copyr-rec))))
 		       (licensed-file-copyright-list rec))
 		  "\n")
-		 (licensed-file-license rec))
+		 (licensed-file-license->string rec))
 	 (format port
 		 "Copyright:\n~a~%License: ~a~%"
 		 (let ((copr-l (sort-copyrights-by-tag-then-first-4 (licensed-file-copyright-list rec))))
@@ -596,7 +697,7 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 				   (copyright-tag copyr)))))
 			     copr-l)
 			"\n")))
-		 (licensed-file-license rec))
+		 (licensed-file-license->string rec))
 	 ))))
 
 (define (all-output dir-tree exclusion-list)
@@ -613,8 +714,9 @@ returns \"1995,2005\".  Or the empty string for no copyrights."
 (define (default-output dir-tree exclusion-list)
   (let* ((d-hash (hash-for-dir-tree-w-exclusions dir-tree exclusion-list))
 	 (license-freqs (license-counts d-hash (licenses-found d-hash)))
-	 (dominant-lic (caar license-freqs))
-	 (rem-licenses (map car (cdr license-freqs)))
+	 (license-freqs-wo-falses (remove (lambda (x) (eq? (car x) #f)) license-freqs))
+	 (dominant-lic (caar license-freqs-wo-falses))
+	 (rem-licenses (map car (cdr license-freqs-wo-falses)))
 	 )
     (format #t "Files: *~%~a~%"
 	    (licensed-files/license d-hash dominant-lic))
